import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { select,hierarchy, pack,
        json, nest,scaleSequential,
        interpolateMagma,scaleLinear,
        max,csv,scaleOrdinal,range
       } from 'd3';

import PACK from './pack.js'

class App extends React.Component{
     
  constructor(){
    super()
    this.drawPack = this.drawPack.bind(this)
    this.state = {data: [] }
  }
  
  
  drawPack(data,props){
    const getValue = props.value
    const getSort = props.sort
    const color = scaleSequential([8, 0],interpolateMagma)
    const selection = select(this.node)
    const height = 400
    const width = 400
    console.log(data)
    const padding = {left:10,right:10,top:10,bottom:10}
    
    const circle =  pack().size([width-2,height-2])
                          .padding(3)
    const meta = hierarchy(data).sum(getValue)
    														.sort(getSort)
    console.log('meta',meta)
    const metaCircle = circle(meta)
  
    console.log('root',metaCircle.descendants().slice(1))
    const groupHeight = nest().key(d=> d.height).entries(metaCircle.descendants())
    console.log('groupHeight',groupHeight.filter(x=>x.key =="0"))
    
    const radius = scaleLinear().domain([0,metaCircle.r]).range([0,metaCircle.r])
    const xmax = max(groupHeight.filter(x=>x.key =="0")[0].values.map(x => x.x))
    const ymax = max(groupHeight.filter(x=>x.key =="0")[0].values.map(x => x.x))
    const xvalue = scaleLinear().domain([0,xmax]).range([0,xmax])
    const yvalue = scaleLinear().domain([0,ymax]).range([0,ymax])
    
    const node = selection.selectAll('g')
    											.data(groupHeight)
    											.enter().append('g')
                          .selectAll('g')
                          .data(d=>d.values)
                          .enter().append('g')
                          .attr('transform',d=>`translate(${xvalue(d.x)},${yvalue(d.y)})`)
                          .attr('id',d=>{console.log(d);return d.data.name;})
    
    //console.log(node)
    node.append('circle').attr('r',d=>radius(d.r)).attr('fill',d=> color(d.height) )
      
    const leaf = node.filter(d => !d.children);
    
    const branch = node.filter(d => d.children);
    leaf.select('circle[fill]').remove()
    
    branch.on("mouseover", function() { select(this).attr("stroke", "#000"); })
          .on("mouseout", function() { select(this).attr("stroke", null); })
    
    
    if(props.image){
      const clipId = scaleOrdinal().domain(props.ids)
      														.range(range(props.ids.length))
      leaf.append('defs').append('clipPath')
          .attr('id',d=> clipId(d.data.celebrity) )
          .append('circle')
          .attr('r',d=>radius(d.r))
      leaf.append('image')
          .attr('clip-path',d => `url(#${clipId(d.data.celebrity)})`)
          .attr('href',d=>d.data.link )
          .attr('height',d=>3*d.r )
          .attr('width',d=>3*d.r )
          .attr('x',d=>-1.5*d.r)
          .attr('y',d=>-1.5*d.r)
      
    }
  }
  
  render(){
   
    
    csv('celebrity.csv').then(table => {
      //this.drawPack(d)
      
      this.setState({data:table})
      
      table.map(d=> d.sample = +d.sample)
      
      console.log('table',table)
      
      const tree = d3.nest()
                  .key(function(d) { return d.category; })
                  .key(function(d) { return d.gender; })
                  .entries
                  (table);
      
      const rename = (tree) => {
         tree.forEach(d=> {
            Object.keys(d).forEach(k=>{
              if(k == 'key'){
                 d['name'] = d['key']
                 delete d['key']
              }else{
                 d['children'] = d['values']
                 delete d['values']
                 
              } 
           })
           if(d.children)rename(d.children)
         })
         return tree;
       }   
        let data = {}
        data['name'] = 'celebrities'
        data['children'] = rename(tree)
        console.log(data)
        console.log(hierarchy(data))
        this.drawPack(data,{value:d=>d.sample,
                            sort:(a,b)=>{ b.sample-a.sample },
                            image:true,
                            ids: table.map(d => d.celebrity)
                           })
       
    })
    
    return (<div>
              <PACK data = {this.state.data} />
            </div>
           );
  }
}
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
